%define releasebase 1 
%define nbase	petsc	
%define prefix /opt/%{nbase}/%{version}/%{compilertag}/%{mpitag}/%{blastag}
%define	version	3.5.3
%define buildingdir %{_builddir}/%{nbase}-%{version}
%define modulepath /opt/Modules/modulefiles/

#Set this to the machine on which you build ATLAS

# allow --with[out] <feature> at rpm command line build
%{?_with_gnu4: %{expand: %%define use_gnu4 1}}
%{?_with_pgi: %{expand: %%define use_pgi 1}}
%{?_with_intel: %{expand: %%define use_intel 1}}
%{?_with_pathsc: %{expand: %%define use_pathsc 1}}
# possible MPI's
%{?_with_mpich2: %{expand: %%define use_mpich2 1}}
%{?_with_mpich2omx: %{expand: %%define use_mpich2omx 1}}
%{?_with_ompi: %{expand: %%define use_ompi 1}}
# possible build tags
%{?_with_basement: %{expand: %%define as_basement 1}}
%{?_with_limulus: %{expand: %%define as_limulus 1}}
# possible build tags
%{?_with_atlas: %{expand: %%define as_atlas 1}}
%{?_with_openblas: %{expand: %%define as_openblas 1}}
#pick the target
%if "1" == "%{as_basement}"
%define targetbase Basement
%else
%define targetbase Limulus
%endif
# pick the compiler
%if "1" == "%{use_pgi}"
%define fflags -O2
%define cflags -O2
%define compilertag pgi
%endif
%if "1" == "%{use_intel}"
%define fflags -O2
%define cflags -O2
%define compilertag intel
%endif
%if "1" == "%{use_pathsc}"
%define compilertag pathsc
%endif
%if "1" == "%{use_gnu3}"
%define compilertag gnu3
%define fflags -O2
%define cflags -O2
%endif
%if "1" == "%{use_gnu4}"
%define compilertag gnu4
%endif
%define fflags -O2
%define cflags -O2
# select MPI
%if "1" == "%{use_ompi}"
%define mpitag openmpi
%define mpiversion 1.8.4
%endif
%if "1" == "%{use_mpich2omx}"
%define mpitag mpich-omx
%define mpiversion 3.1.3
%endif
%if "1" == "%{use_mpich2}"
%define mpitag mpich
%define mpiversion 3.1.3
%endif
# select BLAS libarary
# Set the cpuname to the cpu for which the
# libaray was compiled. (gcc-4.4 does not support haswell)
%if "1" == "%{as_openblas}"
%define blasconfline --with-blas-lapack-lib=$OPENBLAS_PATH/libopenblas.a
%define blastag openblas
%define blasver 0.2.13
%define cpuname haswell
%else
%define blasconfline --with-blas-lapack-dir=$ATLAS_PATH
%define blastag atlas 
%define blasver 3.11.31
%define cpuname ivybridge
%endif

BuildRequires: lapack-%{compilertag}, blas-%{compilertag}, modules


%global _compiler_load  module load %{compilertag}
%global _compiler_unload  module unload %{compilertag}

%global _blas_load  module load %{blastag}/%{blasver}/%{compilertag}/%{cpuname};
%global _blas_unload  module unload %{blastag}/%{blasver}/%{compilertag}/%{cpuname};

%global _mpi_load module load %{mpitag}/%{mpiversion}/%{compilertag};
%global _mpi_unload  module unload %{mpitag}/%{mpiversion}/%{compilertag};


Summary:	Portable Extensible Toolkit for Scientific Computation
Name:		%{nbase}-%{mpitag}-%{blastag}-%{compilertag}
Version:	%{version}
Release: %{releasebase}.%{targetbase}
License:	BSD style
Group:	  %{targetbase}/Libraries	
URL:		http://www-fp.mcs.anl.gov/petsc/
Source:		%{nbase}-%{version}.tar.gz
Requires:	modules
Provides:	%{name}-%{version}
BuildRoot:	%{_tmppath}/%{nbase}-%{version}
BuildRequires:	binutils, automake, autoconf, texinfo, sed, perl


%description 
PETSc is a suite of data structures and routines for the scalable (parallel)
solution of scientific applications modeled by partial differential equations.
This version uses the MPI (mpich) library.

%prep
rm -rf %{buildroot}
%setup -n %{nbase}-%{version}
#%patch0 -p0

%build
rpm -qf /opt/Modules/bin/modulecmd >& /dev/null
if [ $? -ne 0 ]; then
  echo "This RPM requries the BSC Modules package to build"
fi
. /etc/profile.d/modules.sh
%{_mpi_load}
%{_blas_load}
%{_compiler_load}

CURRENT=`pwd`
export PETSC_DIR=$CURRENT
# built the static lib
./configure --prefix=%{prefix} --with-mpi-dir=$MPIPATH \
 --with-debugging=0  \
 --with-single-library=1  \
 --with-shared-libraries=0 \
 --with-pic=1 \
 --FFLAGS=%{fflags} \
 --CFLAGS=%{cflags} \
 %{blasconfline}

echo $PETSC_DIR
make PETSC_DIR=$CURRENT PETSC_ARCH=arch-linux2-c-opt all
# build the shared libs (is there an ption setting to combine these?)
./configure --prefix=%{prefix} --with-mpi-dir=$MPIPATH \
 --with-debugging=0  \
 --with-single-library=1  \
 --with-shared-libraries=1 \
 --with-pic=1 \
 --FFLAGS=%{fflags} \
 --CFLAGS=%{cflags} \
 %{blasconfline}

echo $PETSC_DIR
make PETSC_DIR=$CURRENT PETSC_ARCH=arch-linux2-c-opt all


%{_mpi_unload}
%{_blas_unload}

%{_compiler_unload}

%install
# fix the /usr/local/bin/bash
sed --in-place 's!/usr/local/bin/bash!/bin/bash!' bin/mpiexec.llrun
sed --in-place 's!/usr/local/bin/bash!/bin/bash!' bin/mpiexec.poe
sed --in-place 's!/usr/local/bin/bash!/bin/bash!' bin/mpiexec.prun
mkdir -p %{buildroot}/%{prefix}
CURRENT=`pwd`
make PETSC_DIR=$CURRENT PETSC_ARCH=arch-linux2-c-opt DESTDIR=%{buildroot}/%{prefix} install
# remove config.log so RPM does not complain
/bin/rm %{buildroot}/%{prefix}/conf/configure.log*
# create /etc/ld.so.conf.d/
echo "Creating /opt//etc/ld.so.conf.d/%{name} file"
mkdir -p %{buildroot}/opt/etc/ld.so.conf.d
cat <<EOF>> %{buildroot}/opt/etc/ld.so.conf.d/%{name}-%{version}.conf
%{prefix}/lib
EOF

#create the module file
mkdir -p %{buildroot}/%{modulepath}/%{nbase}/%{version}/%{compilertag}/%{mpitag}
cat <<EOF >%{buildroot}/%{modulepath}/%{nbase}/%{version}/%{compilertag}/%{mpitag}/%{blastag}
#%Module1.0#######################################################################
## %{nbase} modulefile
##

proc ModulesHelp { } {
        puts stderr "\tThis module loads the appropriate environment for "
        puts stderr "\tusing %{name}  version %{version}.\n"
}
prereq %{mpitag}/%{version}/%{compilertag}

module-whatis "%{name} %{version}"
module-log error none

set petscroot %{prefix}

if [ module-info mode load ] {
  prepend-path  LD_LIBRARY_PATH \$petscroot/lib
  setenv PETSC_DIR \$petscroot
  setenv PETSC_PATH \$petscroot/lib
  setenv PETSC_INCLUDE \$petscroot/include
}
if [ module-info mode remove ] {
  remove-path LD_LIBRARY_PATH \$petscroot/lib
  unsetenv PETSC_DIR 
  unsetenv PETSC_PATH
  unsetenv PETSC_INCLUDE
}
EOF
%post
/sbin/ldconfig 2>/dev/null

%postun 
/sbin/ldconfig 2>/dev/null

%files 
%defattr(-,root,root) 
%doc  docs/ 

%{prefix}
%{modulepath}/%{nbase}/%{version}/%{compilertag}/%{mpitag}/%{blastag}
/opt/%{_sysconfdir}/ld.so.conf.d/%{name}-%{version}.conf


%changelog
* Wed Jan 28 2015  Douglas Eadline <deadline@basement-supercomputing.com>
- updated to version 3.5.2, build both static and shared single library
* Fri Dec 12 2012  Douglas Eadline <deadline@basement-supercomputing.com>
- added /opt/etc/ld.so.conf.d entry
* Wed Oct 10 2012 - Douglas Eadline  <deadline@basement-supercomputing.com>
- created

